NOVA runtime environment (NRE)
==============================

NRE is a runtime environment for the microhypervisor NOVA. It can be used to
build a microkernel-based system, for virtualization, or a mix of both.

NRE is supposed to be the successor of NUL (NOVA UserLand). It has the goal
to be much simpler and cleaner than NUL, because NUL is quite complicated and
difficult to maintain. This is achieved by providing easy-to-use abstractions,
using C++ exception handling and the well known pattern RAII.
Another important difference to NUL is that each service (typically a
component that would be a single or a group of host-drivers in NUL) runs in
its own protection domain to make it more reliable and reduce the dependencies
between them. Additionally, the architecture x86_64 (besides x86_32) is
supported from the beginning.


Getting started:
----------------

Please take the following steps:

1.  Go into the directory cross/.
2.  a) Try to find a precompiled cross compiler for your system by executing
    `download.sh <arch>`, where `<arch>` is either `x86_32` or `x86_64`.
    It will download the compiler into `/tmp/nre-cross-<arch>`. There are only
    precompiled versions for certain host platforms, of course. If you are
    uncertain what to choose, try Ubuntu 18.04. It should work for most
    systems, including newer ones.  
    b) If and only if there is no precompiled cross-compiler that works for
    your system, execute `build.sh <arch>`, where `<arch>` is either `x86_32`
    or `x86_64`. This builds the cross compiler. Thus, it will take a few
    minutes.
3.  Choose the target and build-type that you want to use:  
    `export NRE_TARGET=(x86_32|x86_64) NRE_BUILD=(debug|release)`
4.  Go into the directory nre. You'll find the convenience script b, which is
    responsible for building everything that needs to be build and executing
    the specified command. So, simply do a `./b` to build everything. It will
    build NOVA and NRE. Please run `./b -h` to see the help of the script.
    For a quick start use `./b qemu boot/hello` to run the hello application
    in qemu.


Debugging:
----------

In case you experience a problem and would like to debug it, here are a few
general suggestions on how to hunt it down:

* Do always use the debug build of NRE to find errors, as long as they occur in
  debug mode.
* You can enable/disable logging output in nre/include/Logging.h. Especially,
  it might be helpful to turn on EXCEPTIONS in order to see every thrown
  exception.
* It might also help to enable logging in NOVA by enabling some log-types in
  kernel/nova/include/stdio.h.
* To remote debug the system running in qemu via gdb, you can use
  `./b dbg=<app> <bootscript>`. This will start `<bootscript>` and use the
  debugging symbols of `<app>`. Since every NRE application is linked to a
  different address you shouldn't interfere with others.
* If you encounter a problem, you'll often see a backtrace that looks like this:
  
  > [4] Backtrace:  
  > [4] 0000:0003:0030:a4d2  
  > [4] 0000:0003:0030:2210  
  > [4] 0000:0003:0030:fd5c  
  
  You can start `./b trace=<app>`, where `<app>` is e.g. `root`, and simply paste
  the backtrace in your terminal to decode it.

